unit DLUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ScktComp;

type
  TDLForm = class(TForm)
    ClientSock: TClientSocket;
    ProgLabel: TLabel;
    CancelButton: TButton;
    procedure FormResize(Sender: TObject);
    procedure ClientSockConnect(Sender: TObject; Socket: TCustomWinSocket);
    procedure ClientSockDisconnect(Sender: TObject; Socket: TCustomWinSocket);
    procedure ClientSockError(Sender: TObject; Socket: TCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
    procedure ClientSockReadHeaders(Sender: TObject; Socket: TCustomWinSocket);
    procedure ClientSockReadContent(Sender: TObject; Socket: TCustomWinSocket);
    procedure FormCreate(Sender: TObject);
    procedure ProgListChange(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
  private
    procedure IdleSpawnWebInstall(Sender: TObject; var Done: Boolean);
    { Private declarations }
  public
    procedure Go(Host, Path: String);
    { Public declarations }
  end;

var
  DLForm: TDLForm;





implementation

{$R *.DFM}

uses
  ShellAPI, TintProgresses;

const
  WebInstallSizeGuess = 794624;

var
  MyHost, MyPath, TempPath: String;
  Progress: TTintProgress;
  ProgList: TStringList;
  RecBuf: String;
  SaveStream: TFileStream;



function StringReplaceC(const S: String; const OldChar, NewChar: Char): String;
var
  I: Integer;

begin
  Result := S;
  for I := 1 to Length(Result) do
    if Result[I] = OldChar then
      Result[I] := NewChar
  ;
end;

function QueryTempPath: String;
var
  L: Integer;

begin
  L := GetTempPath(0, nil);
  SetString(Result, nil, L);
  L := GetTempPath(L, @Result[1]);
  SetLength(Result, L);
end;

procedure TDLForm.FormResize(Sender: TObject);
var
  H, H2, HP, TP: Integer;

begin
  if not Assigned(Progress) then Exit;
  //W := ClientWidth;
  H := ClientHeight;
  H2 := H div 2;
  HP := Progress.Height;
  TP := H2 - (HP div 2);
  ProgLabel.Height := TP;
  Progress.Top := TP;
  CancelButton.Top := TP + HP;
end;

procedure TDLForm.ProgListChange(Sender: TObject);
begin
  ProgLabel.Caption := ProgList.Text;
end;

procedure TDLForm.FormCreate(Sender: TObject);
begin
  ProgList := TStringList.Create;
  ProgList.OnChange := ProgListChange;

  Progress := TTintProgress.Create(Self);
  Progress.Visible := False;
  Progress.Parent := Self;
  Progress.Min := 0;
  Progress.Max := 1;
  Progress.Position := 0;
  Progress.BorderWidth := 1;
  Progress.Color := clBlack;
  Progress.Left := 0;
  Progress.Width := ClientWidth;
  Progress.Anchors := [akLeft, akTop, akRight];
  FormResize(nil);

  TempPath := IncludeTrailingBackslash(QueryTempPath);
end;

procedure TDLForm.Go(Host, Path: String);
begin
  //Visible := True;
  MyHost := Host;
  MyPath := Path;

  while DeleteFile(TempPath + 'WebInstall.exe') do ;
  ProgList.Add('Connecting...');
  ClientSock.OnRead := ClientSockReadHeaders;
  ClientSock.Host := MyHost;
  ClientSock.Open;
  ShowModal;
end;

procedure TDLForm.ClientSockConnect(Sender: TObject; Socket: TCustomWinSocket);
var
  I: Integer;

begin
  I := ProgList.Count - 1;
  ProgList[I] := 'Connected.';
  ProgList.Add('Downloading web installer...');
  //Request web installer.
  Socket.SendText('GET ' + MyPath + ' HTTP/1.1'#13#10
                + 'Host: ' + MyHost + #13#10
                + 'Connection: close'#13#10
                + #13#10
  );
end;

procedure TDLForm.ClientSockReadHeaders(Sender: TObject; Socket: TCustomWinSocket);
var
  I: Integer;
  HeadList: TStringList;
  SizeBuf: String;

begin
  RecBuf := RecBuf + Socket.ReceiveText;
  I := Pos(#10#13, RecBuf);
  if I <> 0 then begin //Searching for CRLFCRLF by matching LFCR
    ClientSock.OnRead := ClientSockReadContent;
    HeadList := TStringList.Create;
    HeadList.Text := StringReplaceC(Copy(RecBuf, 1, I - 2), ':', '=');
    RecBuf := Copy(RecBuf, I + 3, MAXINT);
    try
      SaveStream := TFileStream.Create(TempPath + 'WebInstall.exe', fmCreate);
    except
      ClientSock.Close;
      Application.MessageBox('A file error occurred. This may be due to open programs; close any open programs and try again.', 'File Error', MB_ICONEXCLAMATION);
      Close;
      Exit;
    end;
    SaveStream.Write(RecBuf[1], Length(RecBuf));
    RecBuf := '';
    SizeBuf := Trim(HeadList.Values['Content-Length']);
    if SizeBuf = '' then
      Progress.Max := WebInstallSizeGuess
    else try
      Progress.Max := StrToInt(SizeBuf);
    except
      Progress.Max := WebInstallSizeGuess;
    end;
    Progress.Visible := True;
  end;
end;

procedure TDLForm.ClientSockReadContent(Sender: TObject; Socket: TCustomWinSocket);
var
  SBuf: String;

begin
  SBuf := Socket.ReceiveText;
  SaveStream.Write(SBuf[1], Length(SBuf));
  Progress.Position := Progress.Position + Length(SBuf);
  SBuf := '';
end;

procedure TDLForm.ClientSockDisconnect(Sender: TObject;Socket: TCustomWinSocket);
begin
  Application.OnIdle := IdleSpawnWebInstall;
end;

procedure TDLForm.ClientSockError(Sender: TObject; Socket: TCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
begin
  ClientSock.Close;
  Application.MessageBox(PChar('Network error ' + IntToStr(ErrorCode) + ' occurred. Check your network connection and try again.'), 'Network Error', MB_ICONEXCLAMATION);
  Close;
end;

procedure TDLForm.IdleSpawnWebInstall(Sender: TObject; var Done: Boolean);
begin
  Application.OnIdle := nil;

  if not Assigned(SaveStream) then Exit;  
  SaveStream.Free;
  ShellExecute(0, nil, PChar(TempPath + 'WebInstall.exe'), nil, nil, SW_SHOWNORMAL);
  Close;
end;

procedure TDLForm.CancelButtonClick(Sender: TObject);
begin
  //
end;

end.
