unit TintProgresses;

interface

uses
  Windows, SysUtils, Classes, Controls, ComCtrls, Graphics;

type
  TTintProgress = class(TCustomControl)
  protected
    FMin: Integer;
    FMax: Integer;
    FPosition: Integer;
    FOrientation: TProgressBarOrientation;
    FBorderWidth: Integer;
    FTint: array[0..4] of TColor;
    procedure SetMin(Value: Integer);
    procedure SetMax(Value: Integer);
    procedure SetPosition(Value: Integer);
    procedure SetOrientation(Value: TProgressBarOrientation);
    procedure SetBorderWidth(Value: Integer);
    procedure SetTint(Value: TColor);
    function GetTints(Index: Integer): TColor;
    procedure SetTints(Index: Integer; Value: TColor);
    procedure Paint; override;
  published
    property Tint: TColor read FTint[2] write SetTint default $FF0000;
  public
    constructor Create(AOwner: TComponent); override;
    property Tints[Index: Integer]: TColor read GetTints write SetTints;
  published
    property Min: Integer read FMin write SetMin;
    property Max: Integer read FMax write SetMax;
    property Orientation: TProgressBarOrientation read FOrientation write SetOrientation default pbHorizontal;
    property Position: Integer read FPosition write SetPosition default 0;
    property BorderWidth: Integer read FBorderWidth write SetBorderWidth default 1;
  published
    property Align;
    property Anchors;
    property Color;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Hint;
    property Constraints;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;





implementation

uses
  Math;

type
  TRGBFColor = packed record
  case Boolean of
    False: (R, G, B, F: Byte);
    True: (Color: TColor);
  end;

function Fade(A, B: TColor; F: Single): TColor;
var
  AB: TRGBFColor absolute A;
  BB: TRGBFColor absolute B;
  RB: TRGBFColor absolute Result;
  nF: Single;

begin
  nF := 1 - F;
  RB.R := Round(AB.R * nF + BB.R * F);
  RB.G := Round(AB.G * nF + BB.G * F);
  RB.B := Round(AB.B * nF + BB.B * F);
  RB.F := 0;
end;

{ TTintProgress }

constructor TTintProgress.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 150;
  Height := GetSystemMetrics(SM_CYVSCROLL);
  FMin := 0;
  FMax := 100;
  FOrientation := pbHorizontal;
  FBorderWidth := 1;
  Tint := $FF0000;
end;

function TTintProgress.GetTints(Index: Integer): TColor;
begin
  if (Index < 0) or (Index > 4) then
    raise ERangeError.Create('Index ' + IntToStr(Index) + ' is invalid.')
  ;
  Result := FTint[Index];
end;

procedure TTintProgress.Paint;
var
  R: TRect;

  procedure AddEdge;
  begin
    Dec(R.Right);
    Dec(R.Bottom);
    Canvas.Pen.Color := clBtnShadow;
    Canvas.MoveTo(R.Left    , R.Top    );
    Canvas.LineTo(R.Right   , R.Top    );
    Canvas.MoveTo(R.Left    , R.Top + 1);
    Canvas.LineTo(R.Left    , R.Bottom );

    Canvas.Pen.Color := clBtnHighlight;
    Canvas.MoveTo(R.Right    , R.Top + 1);
    Canvas.LineTo(R.Right    , R.Bottom );
    Canvas.MoveTo(R.Left  + 1, R.Bottom );
    Canvas.LineTo(R.Right + 1, R.Bottom );
    Canvas.Pixels[R.Left     , R.Bottom ] := clBtnFace;
    Canvas.Pixels[R.Right    , R.Top    ] := clBtnFace;
    Inc(R.Left);
    Inc(R.Top);
  end;

var
  I, D, xD: Integer;
  F: Single;


begin
  R := ClientRect;
  Canvas.Brush.Color := Color;
  Canvas.FillRect(R);
  for I := 1 to BorderWidth do AddEdge;
  if FPosition <= FMin then Exit;
  case FOrientation of
    pbHorizontal: begin
      if FPosition < FMax then begin
        xD := R.Right - R.Left;
        if xD <= 0 then Exit;
        D := Round(xD * FPosition / FMax);
        if D = 0 then
          Inc(D)
        else if D = xD then
          Dec(D)
        ;
        R.Right := R.Left + D;
      end;

      D := R.Bottom - 1 - R.Top;
      for I := 0 to D do begin
        F := I * 4 / D;
        xD := Floor(F);
        F := F - xD;
        if F = 0 then begin
          Canvas.Pen.Color := FTint[xD];
          Canvas.MoveTo(R.Left, R.Top + I);
          Canvas.LineTo(R.Right, R.Top + I);
        end
        else begin
          Canvas.Pen.Color := Fade(FTint[xD], FTint[xD + 1], F);
          Canvas.MoveTo(R.Left, R.Top + I);
          Canvas.LineTo(R.Right, R.Top + I);
        end;
      end;
    end;
    pbVertical: begin
      if FPosition < FMax then begin
      
      end;

      //for I := R.Top to
      //Canvas.Brush.Color := FTint;
      //Canvas.FillRect(R);

    end;
  end;
end;

procedure TTintProgress.SetBorderWidth(Value: Integer);
begin
  if Value <> FBorderWidth then begin
    FBorderWidth := Value;
    Invalidate;
  end;
end;

procedure TTintProgress.SetMax(Value: Integer);
begin
  if Value <> FMax then begin
    FMax := Value;
    Invalidate;
  end;
end;

procedure TTintProgress.SetMin(Value: Integer);
begin
  if Value <> FMin then begin
    FMin := Value;
    Invalidate;
  end;
end;

procedure TTintProgress.SetOrientation(Value: TProgressBarOrientation);
begin
  if Value <> FOrientation then begin
    FOrientation := Value;
    SetBounds(Left, Top, Height, Width);
  end;
end;

procedure TTintProgress.SetPosition(Value: Integer);
begin
  if Value < FMin then
    Value := FMin
  else if Value > FMax then
    Value := FMax
  ;
  if Value <> FPosition then begin
    FPosition := Value;
    Invalidate;
  end;
end;

procedure TTintProgress.SetTint(Value: TColor);
var
  VB: TRGBFColor absolute Value;

begin
  if VB.F <> 0 then Value := GetSysColor(VB.R);

  FTint[0] := Value;
  FTint[1] := $FFFFFF;
  FTint[2] := Value;
  FTint[3] := Fade(Value, 0, 1/3);
  FTint[4] := Fade(Value, 0, 2/3);

  Invalidate;
end;

procedure TTintProgress.SetTints(Index: Integer; Value: TColor);
begin
  if (Index < 0) or (Index > 4) then
    raise ERangeError.Create('Index ' + IntToStr(Index) + ' is invalid.')
  ;
  FTint[Index] := Value;
end;

end.